########################################################################
# updateTestMgrAggregate.pl
#
# 
#
########################################################################

use PjCLib;
use Win32::OLE;
require CQPerlExt;


################
# main
################

# Global variables
$TRUE = 1;
$FALSE = 0;
$latestCollectionDate = "";
$projectToProcess = "all";
$iSQLCommand = "osql -S vm-pjc -d tutorialwarehouse -U admin -P admin -Q ";

# Initialize the date/time variables.
getCurrentDateTime();

# Get the ProjectConsole CQ session variable.
$session = logon();

myMain();
exit;

sub myMain {

    # You must always call the initializePjCLibrary() first before
    # you can use the PjCLib.
    PjCLib->initializePjCLibrary($session, PjCLib->BOOLEAN_TRUE);

    # Initialize variables for tables and fields
    setTableAndFieldNames();

    # If the aggregate test manager measure table is empty, initialize it;
    # Else if there is new collection data, process the aggregate information
    # for the collection; Otherwise, delete the latest collection data in
    # the aggregate table and reinsert new data for the collection date
    if (isTableEmpty() == $TRUE) {
	print "Table is empty\n";
	initTable();
	insLatestCollection();
    } else {
	print "Table is not empty\n";
	my $projResultSet = getProjects();
	while ($projResultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	    $projectToProcess = $projResultSet->GetColumnValue(1);
	    print "project=$projectToProcess\n";
	    if (isNewCollectionData() == $TRUE) {
		print "There is new collection data to process\n";
		if ($latestCollectionDate ne "") {
		    resetPrevLatestCollectionFlag();
		    updateTable();
		    insLatestCollection();
		}
	    } else {
		print "There is no new collection data to process\n";
		deleteLatestCollectionData();
		updateTable();
		insLatestCollection();
	    }
	}
    }
}

sub initTable() {
    my $queryStr;
    my $resultSet;
    my $resultSet2;
    my $entity;
    my $retVal;
    my $totalPlanned;
    my $totalExecuted;
    my $totalPassed;
    my $totalFailed;
    my $totalNotRun;
    my $percentExecPlanned;
    my $project;
    my $iteration;
    my $collectionDate;

    print "Initializing the aggregate test measure table...\n";
    $queryStr =
	"SELECT  SUM($TMEXEC_TESTCASES_PASSED), " .
                "SUM($TMEXEC_TESTCASES_FAILED), " .
                "p.$PROJECT, " . 
                "i.$ITERATION, " .
                "c.$COLLECTIONDATE " .
	"FROM    $TMEXEC_MEASURE tmex, $PROJECT_DIM p, " .
                "$ITERATION_DIM i, $COLLECTIONDATE_DIM c " .
	"WHERE   tmex.$TMEXEC_LATEST_COLLECTION = 0 AND " .
                "tmex.$TMEXEC_ITERATION <> 0 AND " .
                "tmex.$TMEXEC_ITERATION = i.dbid AND " .
		"tmex.$TMEXEC_PROJECT = p.dbid AND " .
		"tmex.$TMEXEC_COLLECTIONDATE = c.dbid " .
 	"GROUP BY p.$PROJECT, i.$ITERATION, c.$COLLECTIONDATE ";

    #print "$queryStr\n";
    $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$totalPassed = $resultSet->GetColumnValue(1);
	$totalFailed = $resultSet->GetColumnValue(2);
	$project = $resultSet->GetColumnValue(3);
	$iteration = $resultSet->GetColumnValue(4);
	$collectionDate = $resultSet->GetColumnValue(5);

	$queryStr = 
	    "SELECT   SUM($TMPLANNED_TESTCASES_PLANNED), " .
                     "p.$PROJECT, " . 
                     "i.$ITERATION, " .
                     "c.$COLLECTIONDATE " .
	    "FROM     $TMPLANNED_MEASURE tmpl, $PROJECT_DIM p, " .
                     "$ITERATION_DIM i, $COLLECTIONDATE_DIM c " .
	    "WHERE    tmpl.$TMPLANNED_ITERATION = i.dbid AND " .
                     "i.$ITERATION = \'$iteration\' AND " .
                     "tmpl.$TMPLANNED_PROJECT = p.dbid AND " .
                     "p.$PROJECT = \'$project\' AND " .
                     "tmpl.$TMPLANNED_COLLECTIONDATE = c.dbid AND " .
                     "c.$COLLECTIONDATE = \'$collectionDate\' " .
	    "GROUP BY p.$PROJECT, i.$ITERATION, c.$COLLECTIONDATE ";

	#print "$queryStr\n";
	$resultSet2 = $session->BuildSQLQuery($queryStr);
	$resultSet2->Execute();

	while ($resultSet2->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	    $totalPlanned = $resultSet2->GetColumnValue(1);
	}
	if ($totalPassed eq "") { $totalPassed = 0 };
	if ($totalFailed eq "") { $totalFailed = 0 };
	$totalExecuted = $totalPassed + $totalFailed;
	if ($totalPlanned eq "") { 
	    $totalPlanned = $totalPassed + $totalFailed;
	}
	$totalNotRun = 
	    $totalPlanned - $totalPassed - $totalFailed;
	if ($totalNotRun < 0) { $totalNotRun = 0};
	if ($totalPlanned > 0) {
	    $percentExecPlanned = 
		sprintf("%.2f", ($totalExecuted/$totalPlanned) * 100);
	} else { $percentExecPlanned = "0.00"; }
	print "\nproject=$project; iteration=$iteration; " .
              "collection=$collectionDate\n" .
              "   planned=$totalPlanned; exec=$totalExecuted; " .
              "passed=$totalPassed; failed=$totalFailed; " .
	      "notRun=$totalNotRun; percentExecPlanned=$percentExecPlanned";

        $entity = $session->BuildEntity("$TMAGGR_MEASURE");
	$entity->SetFieldValue("$TMAGGR_PROJECT", $project);
	$entity->SetFieldValue("$TMAGGR_ITERATION", $iteration);
	$entity->SetFieldValue("$TMAGGR_COLLECTIONDATE", $collectionDate);
	$entity->SetFieldValue("$TMAGGR_LATEST_COLLECTION", "0");
	$entity->SetFieldValue("$TMAGGR_TOTAL_PLANNED", "$totalPlanned");
	$entity->SetFieldValue("$TMAGGR_TOTAL_EXECUTED", "$totalExecuted");
	$entity->SetFieldValue("$TMAGGR_TOTAL_PASSED", "$totalPassed");
	$entity->SetFieldValue("$TMAGGR_TOTAL_FAILED", "$totalFailed");
	$entity->SetFieldValue("$TMAGGR_TOTAL_NOTRUN", "$totalNotRun");
	$entity->SetFieldValue("$TMAGGR_PERCENT_EXEC_PLANNED", 
			       "$percentExecPlanned");

	$retVal = $entity->Validate();
	if ($retVal ne "") { print "Error during Validating: $retVal\n"; }
	$retVal = $entity->Commit();
	if ($retVal ne "") { print "Error during Commit: $retVal\n"; }
    }
}

sub updateTable() {
    my $queryStr;
    my $resultSet;
    my $resultSet2;
    my $entity;
    my $retVal;
    my $totalPlanned;
    my $totalExecuted;
    my $totalPassed;
    my $totalFailed;
    my $totalNotRun;
    my $percentExecPlanned;
    my $project;
    my $iteration;
    my $collectionDate;

    print "Updating the aggregate test measure table after " .
	  "$latestCollectionDate...\n";
    $queryStr =
	"SELECT  SUM($TMEXEC_TESTCASES_PASSED), " .
                "SUM($TMEXEC_TESTCASES_FAILED), " .
                "i.$ITERATION, " .
                "c.$COLLECTIONDATE " .
	"FROM    $TMEXEC_MEASURE tmex, $PROJECT_DIM p, " .
                "$ITERATION_DIM i, $COLLECTIONDATE_DIM c " .
	"WHERE   tmex.$TMEXEC_LATEST_COLLECTION = 0 AND " .
                "tmex.$TMEXEC_ITERATION <> 0 AND " .
                "tmex.$TMEXEC_ITERATION = i.dbid AND " .
		"tmex.$TMEXEC_PROJECT = p.dbid AND " .
                "p.$PROJECT = \'$projectToProcess\' AND " .
		"tmex.$TMEXEC_COLLECTIONDATE = c.dbid AND " .
		"c.$COLLECTIONDATE > \'$latestCollectionDate\' " .
 	"GROUP BY i.$ITERATION, c.$COLLECTIONDATE ";

    #print "$queryStr\n";
    $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$totalPassed = $resultSet->GetColumnValue(1);
	$totalFailed = $resultSet->GetColumnValue(2);
	$project = $projectToProcess;
	$iteration = $resultSet->GetColumnValue(3);
	$collectionDate = $resultSet->GetColumnValue(4);

	$queryStr = 
	    "SELECT   SUM($TMPLANNED_TESTCASES_PLANNED), " .
                     "p.$PROJECT, " . 
                     "i.$ITERATION, " .
                     "c.$COLLECTIONDATE " .
	    "FROM    $TMPLANNED_MEASURE tmpl, $PROJECT_DIM p, " .
                    "$ITERATION_DIM i, $COLLECTIONDATE_DIM c " .
	    "WHERE   tmpl.$TMPLANNED_ITERATION = i.dbid AND " .
                    "i.$ITERATION = \'$iteration\' AND " .
                    "tmpl.$TMPLANNED_PROJECT = p.dbid AND " .
                    "p.$PROJECT = \'$project\' AND " .
                    "tmpl.$TMPLANNED_COLLECTIONDATE = c.dbid AND " .
                    "c.$COLLECTIONDATE = \'$collectionDate\' " .
	    "GROUP BY p.$PROJECT, i.$ITERATION, c.$COLLECTIONDATE ";

	#print "$queryStr\n";
	$resultSet2 = $session->BuildSQLQuery($queryStr);
	$resultSet2->Execute();

	while ($resultSet2->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	    $totalPlanned = $resultSet2->GetColumnValue(1);
	}
	if ($totalPassed eq "") { $totalPassed = 0 };
	if ($totalFailed eq "") { $totalFailed = 0 };
	$totalExecuted = $totalPassed + $totalFailed;
	if ($totalPlanned eq "") { 
	    $totalPlanned = $totalPassed + $totalFailed;
	}
	$totalNotRun = 
	    $totalPlanned - $totalPassed - $totalFailed;
	if ($totalNotRun < 0) { $totalNotRun = 0};
	if ($totalPlanned > 0) {
	    $percentExecPlanned = 
		sprintf("%.2f", ($totalExecuted/$totalPlanned) * 100);
	} else { $percentExecPlanned = "0.00"; }
	print "\nproject=$project; iteration=$iteration; " .
              "collection=$collectionDate\n" .
              "   planned=$totalPlanned; exec=$totalExecuted; " .
              "passed=$totalPassed; failed=$totalFailed; " .
	      "notRun=$totalNotRun; percentExecPlanned=$percentExecPlanned";

        $entity = $session->BuildEntity("$TMAGGR_MEASURE");
	$entity->SetFieldValue("$TMAGGR_PROJECT", $project);
	$entity->SetFieldValue("$TMAGGR_ITERATION", $iteration);
	$entity->SetFieldValue("$TMAGGR_COLLECTIONDATE", $collectionDate);
	$entity->SetFieldValue("$TMAGGR_LATEST_COLLECTION", "0");
	$entity->SetFieldValue("$TMAGGR_TOTAL_PLANNED", "$totalPlanned");
	$entity->SetFieldValue("$TMAGGR_TOTAL_EXECUTED", "$totalExecuted");
	$entity->SetFieldValue("$TMAGGR_TOTAL_PASSED", "$totalPassed");
	$entity->SetFieldValue("$TMAGGR_TOTAL_FAILED", "$totalFailed");
	$entity->SetFieldValue("$TMAGGR_TOTAL_NOTRUN", "$totalNotRun");
	$entity->SetFieldValue("$TMAGGR_PERCENT_EXEC_PLANNED", 
			       "$percentExecPlanned");

	$retVal = $entity->Validate();
	if ($retVal ne "") { print "Error during Validating: $retVal\n"; }
	$retVal = $entity->Commit();
	if ($retVal ne "") { print "Error during Commit: $retVal\n"; }
    }
}

sub insLatestCollection() {
    my $queryStr;
    my $resultSet;
    my $resultSet2;
    my $entity;
    my $retVal;
    my $totalPlanned;
    my $totalExecuted;
    my $totalPassed;
    my $totalFailed;
    my $totalNotRun;
    my $percentExecPlanned;
    my $project;
    my $iteration;
    my $collectionDate;

    print "Inserting latest collection data...\n";
    $queryStr =
	"SELECT  SUM($TMEXEC_TESTCASES_PASSED), " .
                "SUM($TMEXEC_TESTCASES_FAILED), " .
		"p.$PROJECT, " .
                "i.$ITERATION, " .
                "c.$COLLECTIONDATE " .
	"FROM    $TMEXEC_MEASURE tmex, $PROJECT_DIM p, " .
                "$ITERATION_DIM i, $COLLECTIONDATE_DIM c " .
	"WHERE   tmex.$TMEXEC_LATEST_COLLECTION = 1 AND " .
                "tmex.$TMEXEC_ITERATION <> 0 AND " .
                "tmex.$TMEXEC_ITERATION = i.dbid AND " .
		"tmex.$TMEXEC_COLLECTIONDATE = c.dbid AND " .
		"tmex.$TMEXEC_PROJECT = p.dbid " ;
    if ($projectToProcess ne "all") {
	$queryStr = $queryStr .
	            "AND p.$PROJECT = \'$projectToProcess\' " .
		    "GROUP BY p.$PROJECT, i.$ITERATION, c.$COLLECTIONDATE ";
    } else {
 	$queryStr = $queryStr .
	            "GROUP BY p.$PROJECT, i.$ITERATION, c.$COLLECTIONDATE ";
    }

    #print "$queryStr\n";
    $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    while ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$totalPassed = $resultSet->GetColumnValue(1);
	$totalFailed = $resultSet->GetColumnValue(2);
	$project = $resultSet->GetColumnValue(3);
	$iteration = $resultSet->GetColumnValue(4);
	$collectionDate = $resultSet->GetColumnValue(5);

	$queryStr = 
	    "SELECT   SUM($TMPLANNED_TESTCASES_PLANNED), " .
                     "p.$PROJECT, " . 
                     "i.$ITERATION, " .
                     "c.$COLLECTIONDATE " .
	    "FROM    $TMPLANNED_MEASURE tmpl, $PROJECT_DIM p, " .
                    "$ITERATION_DIM i, $COLLECTIONDATE_DIM c " .
	    "WHERE   tmpl.$TMPLANNED_ITERATION = i.dbid AND " .
                    "i.$ITERATION = \'$iteration\' AND " .
                    "tmpl.$TMPLANNED_PROJECT = p.dbid AND " .
                    "p.$PROJECT = \'$project\' AND " .
                    "tmpl.$TMPLANNED_COLLECTIONDATE = c.dbid AND " .
                    "c.$COLLECTIONDATE = \'$collectionDate\' " .
	    "GROUP BY p.$PROJECT, i.$ITERATION, c.$COLLECTIONDATE ";

	#print "$queryStr\n";
	$resultSet2 = $session->BuildSQLQuery($queryStr);
	$resultSet2->Execute();

	while ($resultSet2->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	    $totalPlanned = $resultSet2->GetColumnValue(1);
	}
	if ($totalPassed eq "") { $totalPassed = 0 };
	if ($totalFailed eq "") { $totalFailed = 0 };
	$totalExecuted = $totalPassed + $totalFailed;
	if ($totalPlanned eq "") { 
	    $totalPlanned = $totalPassed + $totalFailed;
	}
	$totalNotRun = 
	    $totalPlanned - $totalPassed - $totalFailed;
	if ($totalNotRun < 0) { $totalNotRun = 0};
	if ($totalPlanned > 0) {
	    $percentExecPlanned = 
		sprintf("%.2f", ($totalExecuted/$totalPlanned) * 100);
	} else { $percentExecPlanned = "0.00"; }
	print "\nproject=$project; iteration=$iteration; " .
              "collection=$collectionDate\n" .
              "   planned=$totalPlanned; exec=$totalExecuted; " .
              "passed=$totalPassed; failed=$totalFailed; " .
	      "notRun=$totalNotRun; percentExecPlanned=$percentExecPlanned";

        $entity = $session->BuildEntity("$TMAGGR_MEASURE");
	$entity->SetFieldValue("$TMAGGR_PROJECT", $project);
	$entity->SetFieldValue("$TMAGGR_ITERATION", $iteration);
	$entity->SetFieldValue("$TMAGGR_COLLECTIONDATE", $collectionDate);
	$entity->SetFieldValue("$TMAGGR_LATEST_COLLECTION", "1");
	$entity->SetFieldValue("$TMAGGR_TOTAL_PLANNED", "$totalPlanned");
	$entity->SetFieldValue("$TMAGGR_TOTAL_EXECUTED", "$totalExecuted");
	$entity->SetFieldValue("$TMAGGR_TOTAL_PASSED", "$totalPassed");
	$entity->SetFieldValue("$TMAGGR_TOTAL_FAILED", "$totalFailed");
	$entity->SetFieldValue("$TMAGGR_TOTAL_NOTRUN", "$totalNotRun");
	$entity->SetFieldValue("$TMAGGR_PERCENT_EXEC_PLANNED", 
			       "$percentExecPlanned");

	$retVal = $entity->Validate();
	if ($retVal ne "") { print "Error during Validating: $retVal\n"; }
	$retVal = $entity->Commit();
	if ($retVal ne "") { print "Error during Commit: $retVal\n"; }
    }
}

sub resetPrevLatestCollectionFlag() {
    my $updateStr;

    print "Resetting previous latest collection flag...\n";
    $updateStr = 
	"UPDATE $TMAGGR_MEASURE SET $TMAGGR_LATEST_COLLECTION = 0 " .
	"WHERE  $TMAGGR_LATEST_COLLECTION = 1 AND " .
               "$TMAGGR_PROJECT IN ( " . 
	           "SELECT dbid FROM $PROJECT_DIM " .
                   "WHERE  $PROJECT = \'$projectToProcess\') ";

    system("$iSQLCommand \"$updateStr\" " .
           "-o updateTestMgrAggregate_resetPrevLatestCollectionFlag.txt");
}

sub deleteLatestCollectionData() {
    my $deleteStr;

    print "Deleting the latest collection data...\n";
    $deleteStr = 
	"DELETE $TMAGGR_MEASURE " . 
        "WHERE  $TMAGGR_LATEST_COLLECTION = 1 AND " .
               "$TMAGGR_PROJECT IN ( " .
                   "SELECT dbid FROM $PROJECT_DIM " .
                   "WHERE  $PROJECT = \'$projectToProcess\') "  ;
    print "$deleteStr\n";
    system("$iSQLCommand \"$deleteStr\" " .
           "-o updateTestMgrAggregate_deleteLatestCollectionData.txt ");
}


sub isNewCollectionData () {
    my $queryStr;
    my $resultSet;
    my $collectionDate;

    print "Checking if there is new collection data to process...\n";
    $queryStr =
	"SELECT  distinct c.$COLLECTIONDATE " .
	"FROM    $TMEXEC_MEASURE tmex, $COLLECTIONDATE_DIM c, " .
                "$PROJECT_DIM p " .
	"WHERE   tmex.$TMEXEC_LATEST_COLLECTION = 1 AND " .
                "tmex.$TMEXEC_COLLECTIONDATE = c.dbid AND " .
                "tmex.$TMEXEC_PROJECT = p.dbid AND " .
                "p.$PROJECT = \'$projectToProcess\'";

    $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    if ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$collectionDate = $resultSet->GetColumnValue(1);
    }
    $queryStr =
	"SELECT  distinct c.$COLLECTIONDATE " .
	"FROM    $TMAGGR_MEASURE tmag, $COLLECTIONDATE_DIM c, " .
                "$PROJECT_DIM p " .
	"WHERE   tmag.$TMAGGR_LATEST_COLLECTION = 1 AND " .
                "tmag.$TMAGGR_COLLECTIONDATE = c.dbid AND " .
                "tmag.$TMAGGR_PROJECT = p.dbid AND " .
                "p.$PROJECT = \'$projectToProcess\'";

    $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    if ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	$latestCollectionDate = $resultSet->GetColumnValue(1);
	print "$collectionDate $latestCollectionDate\n";
	if ($collectionDate ne $latestCollectionDate) {
	    return $TRUE;   # this is a new collection data
	} else {
	    return $FALSE;  # collection data has already been inserted
	}
    }
    return $TRUE;   # Cannot find latest collection data in the aggregate
                    # test measure table - assume new collection data anyway
}

# Check if the TestManagerAggregate_Measure empty.
sub isTableEmpty() {
    my $queryStr;
    my $resultSet;

    print "Checking if the aggregate test manager table is empty...\n";
    $queryStr = "SELECT count(*) " .
	        "FROM   $TMAGGR_MEASURE " .
		"WHERE  dbid <> 0";
    $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();

    if ($resultSet->MoveNext() == $CQPerlExt::CQ_SUCCESS) {
	if ($resultSet->GetColumnValue(1) > 0) { return $FALSE; }
    }
    return $TRUE ;
}

sub getProjects() {
    my $queryStr;
    my $resultSet;

    print "Get list of projects to process...\n";
    $queryStr =
	"SELECT  distinct p.$PROJECT " .
	"FROM    $TMEXEC_MEASURE tmex, $PROJECT_DIM p " .
	"WHERE   tmex.$TMEXEC_PROJECT = p.dbid ";

    $resultSet = $session->BuildSQLQuery($queryStr);
    $resultSet->Execute();
    return $resultSet;
}

sub setTableAndFieldNames () {

    # Set variables for table and fields of TM_Aggregate_Measure
    $TMAGGR_MEASURE = PjCLib->getPhysicalTableDbName("TM_Aggregate_Measure");
    $TMAGGR_LATEST_COLLECTION = PjCLib->getPhysicalFieldDbName("TM_Aggregate_Measure", "IsLatestCollection");
    $TMAGGR_PROJECT = PjCLib->getPhysicalFieldDbName("TM_Aggregate_Measure", "Project_fk");
    $TMAGGR_ITERATION = PjCLib->getPhysicalFieldDbName("TM_Aggregate_Measure", "TM_Iteration_fk");
    $TMAGGR_COLLECTIONDATE = PjCLib->getPhysicalFieldDbName("TM_Aggregate_Measure", "collectiondate_fk");
    $TMAGGR_CALENDARDATE = PjCLib->getPhysicalFieldDbName("TM_Aggregate_Measure", "calendardate_fk");
    $TMAGGR_TOTAL_PLANNED = PjCLib->getPhysicalFieldDbName("TM_Aggregate_Measure", "Aggr Test Cases Planned");
    $TMAGGR_TOTAL_EXECUTED = PjCLib->getPhysicalFieldDbName("TM_Aggregate_Measure", "Aggr Test Cases Executed");
    $TMAGGR_TOTAL_PASSED = PjCLib->getPhysicalFieldDbName("TM_Aggregate_Measure", "Aggr Test Cases Passed");
    $TMAGGR_TOTAL_FAILED = PjCLib->getPhysicalFieldDbName("TM_Aggregate_Measure", "Aggr Test Cases Failed");
    $TMAGGR_TOTAL_NOTRUN = PjCLib->getPhysicalFieldDbName("TM_Aggregate_Measure", "Aggr Test Cases Not Run");
    $TMAGGR_PERCENT_EXEC_PLANNED = PjCLib->getPhysicalFieldDbName("TM_Aggregate_Measure", "PercentExecVSPlanned");

    # Set variables for table and fields of TM_PlannedTestCases_Measure
    $TMPLANNED_MEASURE = PjCLib->getPhysicalTableDbName("TM_PlannedTestCases_Measure");
    $TMPLANNED_LATEST_COLLECTION = PjCLib->getPhysicalFieldDbName("TM_PlannedTestCases_Measure", "IsLatestCollection");
    $TMPLANNED_PROJECT = PjCLib->getPhysicalFieldDbName("TM_PlannedTestCases_Measure", "Project_fk");
    $TMPLANNED_ITERATION = PjCLib->getPhysicalFieldDbName("TM_PlannedTestCases_Measure", "TestCaseIteration_fk");
    $TMPLANNED_COLLECTIONDATE = PjCLib->getPhysicalFieldDbName("TM_PlannedTestCases_Measure", "collectiondate_fk");
    $TMPLANNED_CALENDARDATE = PjCLib->getPhysicalFieldDbName("TM_PlannedTestCases_Measure", "calendardate_fk");
    $TMPLANNED_TESTCASES_PLANNED = PjCLib->getPhysicalFieldDbName("TM_PlannedTestCases_Measure", "TM Test Cases Planned");

    # Set variables for table and fields of TM_ExecutedTestCases_Measure
    $TMEXEC_MEASURE = PjCLib->getPhysicalTableDbName("TM_ExecutedTestCases_Measure");
    $TMEXEC_LATEST_COLLECTION = PjCLib->getPhysicalFieldDbName("TM_ExecutedTestCases_Measure", "IsLatestCollection");
    $TMEXEC_PROJECT = PjCLib->getPhysicalFieldDbName("TM_ExecutedTestCases_Measure", "Project_fk");
    $TMEXEC_ITERATION = PjCLib->getPhysicalFieldDbName("TM_ExecutedTestCases_Measure", "TestCaseIteration_fk");
    $TMEXEC_COLLECTIONDATE = PjCLib->getPhysicalFieldDbName("TM_ExecutedTestCases_Measure", "collectiondate_fk");
    $TMEXEC_CALENDARDATE = PjCLib->getPhysicalFieldDbName("TM_ExecutedTestCases_Measure", "calendardate_fk");
    $TMEXEC_TESTCASES_PASSED = PjCLib->getPhysicalFieldDbName("TM_ExecutedTestCases_Measure", "TM Test Cases Passed");
    $TMEXEC_TESTCASES_FAILED = PjCLib->getPhysicalFieldDbName("TM_ExecutedTestCases_Measure", "TM Test Cases Failed");
    $TMEXEC_TESTCASES_EXECUTED = PjCLib->getPhysicalFieldDbName("TM_ExecutedTestCases_Measure", "TM Test Cases Executed");

    # Set variables for table and fields of Project_Dim
    $PROJECT_DIM = PjCLib->getPhysicalTableDbName("project_dim");
    $PROJECT = PjCLib->getPhysicalFieldDbName("project_dim", "project");

    # Set variables for table and fields of TestCaseIteration_Dim
    $ITERATION_DIM = PjCLib->getPhysicalTableDbName("TM_Iteration.Name_dim");
    $ITERATION = PjCLib->getPhysicalFieldDbName("TM_Iteration.Name_dim", "Iteration.Name");
    print "$ITERATION_DIM.$ITERATION $TMAGGR_ITERATION\n";

    # Set variables for table and fields of collectiondate
    $COLLECTIONDATE_DIM = PjCLib->getPhysicalTableDbName("collectiondate");
    $COLLECTIONDATE = PjCLib->getPhysicalFieldDbName("collectiondate", "collectiondate_name");

    # Set variables for table and fields of calendardate
    $CALENDARDATE_DIM = PjCLib->getPhysicalTableDbName("calendardate");
    $CALENDARDATE = PjCLib->getPhysicalFieldDbName("calendardate", "calendardate_name");

 }



sub logon {

    #
    # Use the internal DashboardSystem login to get access to the Dashboard database
    #
    my $session = Win32::OLE->new('CLEARQUEST.SESSION');
    my $session = CQPerlExt::CQSession_Build();  
    $session->UserLogon("dashboardsystem", "dashboardsystem", "tstd1", "Dashboard");
    print "connected\n";
    return $session;
}

# get date and set the global variables $currentDate and $currentTime
sub getCurrentDateTime {
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);

    $mon = $mon + 1;
    $year = $year + 1900;

    if ($mday<10) {$mday="0$mday"; }  # add leading 0
    if ($mon<10) {$mon="0$mon"; }     # add leading 0

    # set the global variable $currentDate
    $currentDate="$year-$mon-$mday";

    # set the global variable $currentTime
    $currentTime = $hour . ":" . $min . ":" . $sec;

}

